package space.kklochko.jdbc_hospital_example.cli.commands.factories;

import lombok.NonNull;
import space.kklochko.jdbc_hospital_example.cli.commands.CommandData;
import space.kklochko.jdbc_hospital_example.cli.commands.CommandEntity;
import space.kklochko.jdbc_hospital_example.db.repositories.AbstractRepository;
import space.kklochko.jdbc_hospital_example.models.Department;
import space.kklochko.jdbc_hospital_example.models.Indicator;
import space.kklochko.jdbc_hospital_example.models.Patient;

import java.lang.reflect.Method;
import java.util.UUID;

public class CommandFactory {
    public <T> CommandEntity create(@NonNull CommandData commandData, @NonNull AbstractRepository<T> repository) {
        try {
            EntityFactory entityFactory = new EntityFactory();

            if(commandData.getName().equals("insert")) {
                if(commandData.getDatatype().equals("indicator")) {
                    Method method = repository.getClass().getMethod("create", Indicator.class);
                    Indicator indicator = (Indicator) entityFactory.create(commandData);
                    return new CommandEntity<Indicator, T>(repository, method, indicator);
                }else if(commandData.getDatatype().equals("patient")) {
                    Method method = repository.getClass().getMethod("create", Patient.class);
                    Patient patient = (Patient) entityFactory.create(commandData);
                    return new CommandEntity<Patient, T>(repository, method, patient);
                }else {
                    Method method = repository.getClass().getMethod("create", Department.class);
                    Department department = (Department) entityFactory.create(commandData);
                    return new CommandEntity<Department, T>(repository, method, department);
                }
            }else if(commandData.getName().equals("update")) {
                if(commandData.getDatatype().equals("indicator")) {
                    Method method = repository.getClass().getMethod("update", Indicator.class);
                    Indicator indicator = (Indicator) entityFactory.create(commandData);
                    return new CommandEntity<Indicator, T>(repository, method, indicator);
                }else if(commandData.getDatatype().equals("patient")) {
                    Method method = repository.getClass().getMethod("update", Patient.class);
                    Patient patient = (Patient) entityFactory.create(commandData);
                    return new CommandEntity<Patient, T>(repository, method, patient);
                }else {
                    Method method = repository.getClass().getMethod("update", Department.class);
                    Department department = (Department) entityFactory.create(commandData);
                    return new CommandEntity<Department, T>(repository, method, department);
                }
            }else {
                String uuidString = commandData.arguments.get("id");
                UUID uuid = UUID.fromString(uuidString);

                Method method = repository.getClass().getMethod("delete", UUID.class);
                return new CommandEntity<UUID, T>(repository, method, uuid);
            }
        } catch (NoSuchMethodException e) {
            throw new RuntimeException(e);
        }
    }
}
